@@ -2,27 +2,30 @@ from __future__ import annotations
 
 import os
 import platform
+import random
 import re
 import subprocess as sp
 import sys
+import tokenize
 import traceback
+import uuid
 from argparse import ArgumentParser
+from contextlib import contextmanager
 from io import StringIO
 from pathlib import Path
-from pathlib import PurePosixPath
 from tempfile import TemporaryDirectory
 from typing import Any
+from typing import Callable
+from unittest.mock import patch
 
 from rich.console import Console
 
 from inline_snapshot._exceptions import UsageError
-from inline_snapshot._external import DiscStorage
-from inline_snapshot._problems import report_problems
+from inline_snapshot._snapshot_session import SnapshotSession
 
-from .._change import apply_all
-from .._flags import Flags
-from .._global_state import snapshot_env
-from .._rewrite_code import ChangeRecorder
+from .._global_state import enter_snapshot_context
+from .._global_state import leave_snapshot_context
+from .._global_state import state
 from .._types import Category
 from .._types import Snapshot
 
@@ -51,6 +54,17 @@ def normalize(text):
     return text
 
 
+@contextmanager
+def deterministic_uuid():
+    rd = random.Random(0)
+
+    def f():
+        return uuid.UUID(int=rd.getrandbits(128), version=4)
+
+    with patch("uuid.uuid4", new=f):
+        yield
+
+
 # this code is copied from pytest
 
 # Regex to match the session duration string in the summary: "74.34s".
@@ -77,47 +91,171 @@ def parse_outcomes(lines):
     return {to_plural.get(k, k): v for k, v in ret.items()}
 
 
+conftest_footer = """
+import uuid
+import random
+
+rd = random.Random(0)
+
+def f():
+    return uuid.UUID(int=rd.getrandbits(128), version=4)
+
+uuid.uuid4 = f
+"""
+
+
+@contextmanager
+def chdir(path):
+    cwd = os.getcwd()
+    try:
+        os.chdir(path)
+        yield
+    finally:
+        os.chdir(cwd)
+
+
+@contextmanager
+def change_file(path: Path, map_function):
+    exists = path.exists()
+    if exists:
+        text = path.read_bytes().decode("utf-8")
+    else:
+        text = ""
+
+    path.write_bytes(map_function(text).encode("utf-8"))
+
+    yield
+
+    if exists:
+        path.write_bytes(text.encode("utf-8"))
+    else:
+        path.unlink()
+
+
+@contextmanager
+def temp_environ(**kwargs):
+    original = dict(os.environ)
+    os.environ.update(kwargs)
+    try:
+        yield
+    finally:
+        os.environ.clear()
+        os.environ.update(original)
+
+
+class StopTesting(Exception):
+    pass
+
+
 class Example:
-    def __init__(self, files: str | dict[str, str]):
+    files: dict[str, str | bytes]
+
+    def __init__(self, files: str | bytes | dict[str, str | bytes]):
         """
         Parameters:
-            files: a collection of files where inline-snapshot operates on,
-                   or just a string which will be saved as *test_something.py*.
+            files: a collection of files which are used as your example project,
+                   or just a string which will be saved as *tests/test_something.py*.
         """
-        if isinstance(files, str):
-            files = {"test_something.py": files}
+        if isinstance(files, (str, bytes)):
+            files = {"tests/test_something.py": files}
 
         self.files = files
 
-        self.dump_files()
-
     def dump_files(self):
+        from rich.panel import Panel
+        from rich.text import Text
+
+        console = Console()
+
         for name, content in self.files.items():
-            print(f"file: {name}")
-            print(content)
-            print()
+            if isinstance(content, bytes):
+                content = repr(content)
+            console.print(Panel(Text(content), title=name))
 
     def _write_files(self, dir: Path):
         for name, content in self.files.items():
             filename = dir / name
             filename.parent.mkdir(exist_ok=True, parents=True)
-            filename.write_text(content)
+            if isinstance(content, str):
+                content = content.encode("utf-8")
+            filename.write_bytes(content)
 
     def _read_files(self, dir: Path):
-        storage_dir = dir / ".inline-snapshot"
+
+        def try_read(path: Path) -> str | bytes:
+            code = path.read_bytes()
+            try:
+                return code.decode("utf-8")
+            except UnicodeDecodeError:
+                return code
+
+        def normalize_path(path):
+            return str(path.relative_to(dir)).replace("\\", "/")
 
         return {
-            str(p.relative_to(dir)): p.read_text()
-            for p in [*dir.iterdir(), *dir.rglob("*.py"), *storage_dir.rglob("*")]
+            normalize_path(p): try_read(p)
+            for p in dir.rglob("*")
             if p.is_file()
+            and p.name != ".gitignore"
+            and p.suffix != ".pyc"
+            and ".pytest_cache" not in p.parts
         }
 
-    def with_files(self, extra_files):
-        return Example(self.files | extra_files)
+    def with_files(self, extra_files: dict[str, str | bytes]) -> Example:
+        """
+        Adds extra files to the example.
 
-    def code_change(self, src, dest):
+        Arguments:
+            extra_files: dictionary of filenames and file contents.
+        """
+        return Example({**self.files, **extra_files})
+
+    def read_file(self, filename: str) -> str:
+        """
+        Reads a file from the example.
+
+        Arguments:
+            filename: the filename.
+        """
+        text = self.files[filename]
+        assert isinstance(text, str)
+        return text
+
+    read_text = read_file
+
+    def change_code(self, mapping: Callable[[str], str]) -> Example:
+        """
+        Changes example tests by mapping every file with the given function.
+
+        Arguments:
+            mapping: function to apply to each file's content.
+        """
         return Example(
-            {name: file.replace(src, dest) for name, file in self.files.items()}
+            {
+                name: mapping(text) if isinstance(text, str) else text
+                for name, text in self.files.items()
+            }
+        )
+
+    def replace(self, old_text: str, new_text: str) -> Example:
+        """
+        Changes example tests by replacing old_text with new_text.
+
+        Arguments:
+            old_text: the text to be replaced.
+            new_text: the new text to use instead.
+        """
+        return self.change_code(lambda code: code.replace(old_text, new_text))
+
+    def remove_file(self, filename: str) -> Example:
+        """
+        Removes a file from the example.
+
+        Arguments:
+            filename: the file to be removed.
+        """
+        return Example(
+            {name: file for name, file in self.files.items() if name != filename}
         )
 
     def run_inline(
@@ -128,6 +266,7 @@ class Example:
         changed_files: Snapshot[dict[str, str]] | None = None,
         report: Snapshot[str] | None = None,
         raises: Snapshot[str] | None = None,
+        stderr: Snapshot[str] | None = None,
     ) -> Example:
         """Execute the example files in process and run every `test_*`
         function.
@@ -135,14 +274,14 @@ class Example:
         This is useful for fast test execution.
 
         Parameters:
-            args: inline-snapshot arguments (supports only "--inline-snapshot=fix|create|..." ).
+            args: inline-snapshot arguments (supports only "--inline-snapshot=fix|create|...").
             reported_categories: snapshot of categories which inline-snapshot thinks could be applied.
             changed_files: snapshot of files which are changed by this run.
-            raises: snapshot of the exception which is raised during the test execution.
-                    It is required if your code raises an exception.
+            raises: snapshot of the exception raised during test execution.
+                    Required if your code raises an exception.
 
         Returns:
-            A new Example instance which contains the changed files.
+            A new Example instance containing the changed files.
         """
 
         parser = ArgumentParser()
@@ -163,6 +302,8 @@ class Example:
         )
         parsed_args = parser.parse_args(args)
         flags = (parsed_args.inline_snapshot or "").split(",")
+        self.dump_files()
+        flags = {f for f in flags if f}
 
         with TemporaryDirectory() as dir:
             tmp_path = Path(dir)
@@ -170,17 +311,35 @@ class Example:
             self._write_files(tmp_path)
 
             raised_exception = []
-            with snapshot_env() as state:
-                recorder = ChangeRecorder()
-                state.update_flags = Flags({*flags})
-                state.storage = DiscStorage(tmp_path / ".storage")
+
+            with deterministic_uuid(), chdir(tmp_path), temp_environ(TERM="unknown"):
+                session = SnapshotSession()
+
+                def report_error(message):
+                    raise StopTesting(message)
+
+                snapshot_flags = set()
                 try:
+                    enter_snapshot_context()
+                    session.load_config(
+                        tmp_path / "pyproject.toml",
+                        flags,
+                        parallel_run=False,
+                        error=report_error,
+                        project_root=tmp_path,
+                    )
+
+                    report_output = StringIO()
+                    console = Console(file=report_output, width=80)
+
                     tests_found = False
-                    for filename in tmp_path.glob("*.py"):
+                    for filename in tmp_path.rglob("test_*.py"):
                         globals: dict[str, Any] = {}
-                        print("run> pytest", filename)
+                        print("run> pytest-inline", filename)
+                        with tokenize.open(filename) as f:
+                            code = f.read()
                         exec(
-                            compile(filename.read_text("utf-8"), filename, "exec"),
+                            compile(code, filename, "exec"),
                             globals,
                         )
 
@@ -194,37 +353,32 @@ class Example:
 
                         for v in tests:
                             try:
-                                v()
+
+                                def fail(message):
+                                    console.print(f"FAIL: {message}")
+
+                                session.test_enter()
+                                try:
+                                    v()
+                                finally:
+                                    session.test_exit(fail=fail)
                             except Exception as e:
                                 traceback.print_exc()
                                 raised_exception.append(e)
 
                     if not tests_found:
                         raise UsageError("no test_*() functions in the example")
+
+                    session.show_report(console)
+
+                    for snapshot in state().snapshots.values():
+                        for change in snapshot._changes():
+                            snapshot_flags.add(change.flag)
+
+                except StopTesting as e:
+                    assert stderr == f"ERROR: {e}\n"
                 finally:
-                    state.active = False
-
-                changes = []
-                for snapshot in state.snapshots.values():
-                    changes += snapshot._changes()
-
-                snapshot_flags = {change.flag for change in changes}
-
-                apply_all(
-                    [
-                        change
-                        for change in changes
-                        if change.flag in state.update_flags.to_set()
-                    ],
-                    recorder,
-                )
-                recorder.fix_all()
-
-                report_output = StringIO()
-                console = Console(file=report_output, width=80)
-
-                # TODO: add all the report output here
-                report_problems(lambda: console)
+                    leave_snapshot_context()
 
             if reported_categories is not None:
                 assert sorted(snapshot_flags) == reported_categories
@@ -240,18 +394,27 @@ class Example:
                 assert raises == None
 
             if changed_files is not None:
-                current_files = {}
-
-                for name, content in sorted(self._read_files(tmp_path).items()):
-                    if name not in self.files or self.files[name] != content:
-                        current_files[name] = content
-                assert changed_files == current_files
+                assert changed_files == self._changed_files(tmp_path)
 
             if report is not None:
                 assert report == normalize(report_output.getvalue())
 
             return Example(self._read_files(tmp_path))
 
+    def _changed_files(self, tmp_path):
+        current_files = {}
+        all_current_files = self._read_files(tmp_path)
+
+        for name, content in sorted(all_current_files.items()):
+            if name not in self.files or self.files[name] != content:
+                current_files[name] = content
+
+        for name in self.files:
+            if name not in all_current_files:
+                current_files[name] = None
+
+        return current_files
+
     def run_pytest(
         self,
         args: list[str] = [],
@@ -266,25 +429,27 @@ class Example:
         stdin: bytes = b"",
         outcomes: Snapshot[dict[str, int]] | None = None,
     ) -> Example:
-        """Run pytest with the given args and env variables in an separate
+        """Run pytest with the given args and environment variables in a separate
         process.
 
-        It can be used to test the interaction between your code and pytest, but it is a bit slower than `run_inline`
+        It can be used to test the interaction between your code and pytest, but it is a bit slower than `run_inline`.
 
         Parameters:
             args: pytest arguments like "--inline-snapshot=fix"
             env: dict of environment variables
-            changed_files: snapshot of files which are changed by this run.
+            changed_files: snapshot of files changed by this run.
             report: snapshot of the report at the end of the pytest run.
             stderr: pytest stderr output
-            returncode: snapshot of the pytest returncode.
+            returncode: snapshot of the pytest return code.
 
         Returns:
-            A new Example instance which contains the changed files.
+            A new Example instance containing the changed files.
         """
+        self.dump_files()
 
         with TemporaryDirectory() as dir:
             tmp_path = Path(dir)
+
             self._write_files(tmp_path)
 
             cmd = [sys.executable, "-m", "pytest", *args]
@@ -296,6 +461,7 @@ class Example:
             )
             command_env.pop("CI", None)
             command_env.pop("GITHUB_ACTIONS", None)
+            command_env.pop("PYTEST_XDIST_WORKER", None)
 
             if stdin:
                 # makes Console.is_terminal == True
@@ -303,12 +469,16 @@ class Example:
 
             command_env.update(env)
 
-            result = sp.run(
-                cmd, cwd=tmp_path, capture_output=True, env=command_env, input=stdin
-            )
+            with change_file(
+                tmp_path / "conftest.py", lambda text: text + conftest_footer
+            ):
+                result = sp.run(
+                    cmd, cwd=tmp_path, capture_output=True, env=command_env, input=stdin
+                )
+
+            result_stdout = result.stdout.decode("utf-8")
+            result_stderr = result.stderr.decode("utf-8")
 
-            result_stdout = result.stdout.decode()
-            result_stderr = result.stderr.decode()
             result_returncode = result.returncode
 
             print("run>", *cmd)
@@ -372,12 +542,7 @@ class Example:
                 )
 
             if changed_files is not None:
-                current_files = {}
-
-                for name, content in sorted(self._read_files(tmp_path).items()):
-                    if name not in self.files or self.files[name] != content:
-                        current_files[str(PurePosixPath(*Path(name).parts))] = content
-                assert changed_files == current_files
+                assert changed_files == self._changed_files(tmp_path)
 
             if outcomes is not None:
                 assert outcomes == parse_outcomes(result_stdout.splitlines())
