@@ -8,7 +8,7 @@ import traceback
 from dataclasses import dataclass
 from dataclasses import field
 from pathlib import Path
-from types import SimpleNamespace
+from typing import List
 from typing import Set
 from unittest import mock
 
@@ -17,12 +17,13 @@ import executing
 import pytest
 
 import inline_snapshot._external
+from inline_snapshot._change import ChangeBase
 from inline_snapshot._change import apply_all
 from inline_snapshot._flags import Flags
 from inline_snapshot._format import format_code
+from inline_snapshot._global_state import snapshot_env
 from inline_snapshot._rewrite_code import ChangeRecorder
 from inline_snapshot._types import Category
-from inline_snapshot.testing._example import snapshot_env
 
 pytest_plugins = "pytester"
 
@@ -95,7 +96,7 @@ from inline_snapshot import outsource
 """
             source = prefix + textwrap.dedent(self.source)
 
-            filename.write_text(source, "utf-8")
+            filename.write_bytes(source.encode("utf-8"))
 
             print()
             print("input:")
@@ -104,9 +105,10 @@ from inline_snapshot import outsource
             with snapshot_env() as state:
                 recorder = ChangeRecorder()
                 state.update_flags = flags
-                state.storage = inline_snapshot._external.DiscStorage(
+                state.all_storages["hash"] = inline_snapshot._external.HashStorage(
                     tmp_path / ".storage"
                 )
+                state.config.storage_dir = tmp_path / ".inline-snapshot"
 
                 error = False
 
@@ -120,9 +122,9 @@ from inline_snapshot import outsource
 
                 number_snapshots = len(state.snapshots)
 
-                changes = []
+                changes: List[ChangeBase] = []
                 for snapshot in state.snapshots.values():
-                    changes += snapshot._changes()
+                    changes += list(snapshot._changes())
 
                 snapshot_flags = {change.flag for change in changes}
 
@@ -258,7 +260,7 @@ class RunResult:
 
 
 @pytest.fixture
-def project(pytester):
+def project(pytester):  # pragma: no cover
     class Project:
 
         def __init__(self):
@@ -281,7 +283,7 @@ from inline_snapshot import outsource
 from inline_snapshot import snapshot
 from inline_snapshot import outsource
 """
-            else:
+            else:  # pragma: no cover
                 self.header = ""
 
             header = self.header
@@ -291,10 +293,10 @@ from inline_snapshot import outsource
             source = header + source
             print("write code:")
             print(source)
-            self._filename.write_text(source, "utf-8")
+            self._filename.write_bytes(source.encode("utf-8"))
 
-            (pytester.path / "conftest.py").write_text(
-                """
+            (pytester.path / "conftest.py").write_bytes(
+                b"""
 import datetime
 import pytest
 from freezegun.api import FakeDatetime,FakeDate
@@ -318,22 +320,26 @@ def set_time(freezer):
 
         @property
         def _filename(self):
-            return pytester.path / "test_file.py"
+            (pytester.path / "tests").mkdir(exist_ok=True)
+
+            return pytester.path / "tests" / "test_file.py"
 
         def is_formatted(self):
             code = self._filename.read_text("utf-8")
             return code == format_code(code, self._filename)
 
         def format(self):
-            self._filename.write_text(
-                format_code(self._filename.read_text("utf-8"), self._filename), "utf-8"
+            self._filename.write_bytes(
+                format_code(self._filename.read_text("utf-8"), self._filename).encode(
+                    "utf-8"
+                )
             )
 
         def pyproject(self, source):
             self.write_file("pyproject.toml", source)
 
         def write_file(self, filename, content):
-            (pytester.path / filename).write_text(content, "utf-8")
+            (pytester.path / filename).write_bytes(content.encode("utf-8"))
 
         def storage(self, storage_dir=".inline-snapshot"):
             if os.path.isabs(storage_dir):
@@ -363,6 +369,7 @@ def set_time(freezer):
                 del os.environ["CI"]  # pragma: no cover
 
             os.environ.pop("GITHUB_ACTIONS", None)
+            os.environ.pop("PYTEST_XDIST_WORKER", None)
 
             try:
                 with mock.patch.dict(
@@ -395,9 +402,12 @@ def executing_used(request, monkeypatch):
     if used:
         yield used
     else:
+        real_executing = executing.Source.executing
 
         def fake_executing(frame):
-            return SimpleNamespace(node=None)
+            result = real_executing(frame)
+            result.node = None
+            return result
 
         monkeypatch.setattr(executing.Source, "executing", fake_executing)
         yield used
