@@ -30,7 +30,7 @@ strict=true
     yield f
 
 
-def test_typing(check_typing):
+def test_typing_args(check_typing):
     check_typing(
         """
 from inline_snapshot import snapshot
@@ -38,3 +38,19 @@ snapshot([])
 snapshot({})
     """
     )
+
+
+def test_typing_call(check_typing):
+    check_typing(
+        """
+from inline_snapshot import snapshot,Snapshot
+
+def f(s:Snapshot[int])->None:
+    assert s==5
+
+f(5)
+f(snapshot())
+f(snapshot(5))
+
+    """
+    )
